@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """An HTTP client based on QNetworkAccessManager."""
 
@@ -23,11 +8,11 @@ import functools
 import urllib.parse
 from typing import MutableMapping
 
-from PyQt5.QtCore import pyqtSignal, QObject, QTimer
-from PyQt5.QtNetwork import (QNetworkAccessManager, QNetworkRequest,
+from qutebrowser.qt.core import pyqtSignal, QObject, QTimer
+from qutebrowser.qt.network import (QNetworkAccessManager, QNetworkRequest,
                              QNetworkReply)
 
-from qutebrowser.utils import log
+from qutebrowser.utils import qtlog
 
 
 class HTTPRequest(QNetworkRequest):
@@ -35,8 +20,8 @@ class HTTPRequest(QNetworkRequest):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.setAttribute(QNetworkRequest.RedirectPolicyAttribute,
-                          QNetworkRequest.NoLessSafeRedirectPolicy)
+        self.setAttribute(QNetworkRequest.Attribute.RedirectPolicyAttribute,
+                          QNetworkRequest.RedirectPolicy.NoLessSafeRedirectPolicy)
 
 
 class HTTPClient(QObject):
@@ -61,7 +46,7 @@ class HTTPClient(QObject):
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        with log.disable_qt_msghandler():
+        with qtlog.disable_qt_msghandler():
             # WORKAROUND for a hang when messages are printed, see our
             # NetworkAccessManager subclass for details.
             self._nam = QNetworkAccessManager(self)
@@ -78,7 +63,7 @@ class HTTPClient(QObject):
             data = {}
         encoded_data = urllib.parse.urlencode(data).encode('utf-8')
         request = HTTPRequest(url)
-        request.setHeader(QNetworkRequest.ContentTypeHeader,
+        request.setHeader(QNetworkRequest.KnownHeaders.ContentTypeHeader,
                           'application/x-www-form-urlencoded;charset=utf-8')
         reply = self._nam.post(request, encoded_data)
         self._handle_reply(reply)
@@ -118,7 +103,7 @@ class HTTPClient(QObject):
         if timer is not None:
             timer.stop()
             timer.deleteLater()
-        if reply.error() != QNetworkReply.NoError:
+        if reply.error() != QNetworkReply.NetworkError.NoError:
             self.error.emit(reply.errorString())
             return
         try:
