@@ -1,59 +1,61 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Utilities for IPC with existing instances."""
 
 import os
-import sys
 import time
 import json
 import getpass
 import binascii
 import hashlib
+from typing import Optional
 
-from PyQt5.QtCore import pyqtSignal, pyqtSlot, QObject, Qt
-from PyQt5.QtNetwork import QLocalSocket, QLocalServer, QAbstractSocket
+from qutebrowser.qt.core import pyqtSignal, pyqtSlot, QObject, Qt
+from qutebrowser.qt.network import QLocalSocket, QLocalServer, QAbstractSocket
 
 import qutebrowser
-from qutebrowser.utils import log, usertypes, error, objreg, standarddir
+from qutebrowser.utils import log, usertypes, error, standarddir, utils, debug, qtutils
+from qutebrowser.qt import sip
 
 
 CONNECT_TIMEOUT = 100  # timeout for connecting/disconnecting
 WRITE_TIMEOUT = 1000
 READ_TIMEOUT = 5000
-ATIME_INTERVAL = 60 * 60 * 3 * 1000  # 3 hours
+ATIME_INTERVAL = 5000 * 60  # 5 minutes
 PROTOCOL_VERSION = 1
 
 
-def _get_socketname_legacy(basedir):
-    """Legacy implementation of _get_socketname."""
-    parts = ['qutebrowser', getpass.getuser()]
+# The ipc server instance
+server = None
+
+
+def _get_socketname_windows(basedir):
+    """Get a socketname to use for Windows."""
+    try:
+        username = getpass.getuser()
+    except ImportError:
+        # getpass.getuser() first tries a couple of environment variables. If
+        # none of those are set (i.e., USERNAME is missing), it tries to import
+        # the "pwd" module which is unavailable on Windows.
+        raise Error("Could not find username. This should only happen if "
+                    "there is a bug in the application launching qutebrowser, "
+                    "preventing the USERNAME environment variable from being "
+                    "passed. If you know more about when this happens, please "
+                    "report this to mail@qutebrowser.org.")
+
+    parts = ['qutebrowser', username]
     if basedir is not None:
         md5 = hashlib.md5(basedir.encode('utf-8')).hexdigest()
         parts.append(md5)
     return '-'.join(parts)
 
 
-def _get_socketname(basedir, legacy=False):
+def _get_socketname(basedir):
     """Get a socketname to use."""
-    if legacy or os.name == 'nt':
-        return _get_socketname_legacy(basedir)
+    if utils.is_windows:  # pragma: no cover
+        return _get_socketname_windows(basedir)
 
     parts_to_hash = [getpass.getuser()]
     if basedir is not None:
@@ -62,11 +64,9 @@ def _get_socketname(basedir, legacy=False):
     data_to_hash = '-'.join(parts_to_hash).encode('utf-8')
     md5 = hashlib.md5(data_to_hash).hexdigest()
 
-    target_dir = standarddir.runtime()
-
-    parts = ['ipc']
-    parts.append(md5)
-    return os.path.join(target_dir, '-'.join(parts))
+    prefix = 'i-' if utils.is_mac else 'ipc-'
+    filename = '{}{}'.format(prefix, md5)
+    return os.path.join(standarddir.runtime(), filename)
 
 
 class Error(Exception):
@@ -93,12 +93,12 @@ class SocketError(Error):
         """
         super().__init__()
         self.action = action
-        self.code = socket.error()
-        self.message = socket.errorString()
+        self.code: QLocalSocket.LocalSocketError = socket.error()
+        self.message: str = socket.errorString()
 
     def __str__(self):
-        return "Error while {}: {} (error {})".format(
-            self.action, self.message, self.code)
+        return "Error while {}: {} ({})".format(
+            self.action, self.message, debug.qenum_key(QLocalSocket, self.code))
 
 
 class ListenError(Error):
@@ -110,19 +110,19 @@ class ListenError(Error):
         message: The error message.
     """
 
-    def __init__(self, server):
+    def __init__(self, local_server):
         """Constructor.
 
         Args:
-            server: The QLocalServer which has the error set.
+            local_server: The QLocalServer which has the error set.
         """
         super().__init__()
-        self.code = server.serverError()
-        self.message = server.errorString()
+        self.code: QAbstractSocket.SocketError = local_server.serverError()
+        self.message: str = local_server.errorString()
 
     def __str__(self):
-        return "Error while listening to IPC server: {} (error {})".format(
-            self.message, self.code)
+        return "Error while listening to IPC server: {} ({})".format(
+            self.message, debug.qenum_key(QAbstractSocket, self.code))
 
 
 class AddressInUseError(ListenError):
@@ -140,8 +140,6 @@ class IPCServer(QObject):
         _server: A QLocalServer to accept new connections.
         _socket: The QLocalSocket we're currently connected to.
         _socketname: The socketname to use.
-        _socketopts_ok: Set if using setSocketOptions is working with this
-                        OS/Qt version.
         _atime_timer: Timer to update the atime of the socket regularly.
 
     Signals:
@@ -170,25 +168,29 @@ class IPCServer(QObject):
         self._timer.setInterval(READ_TIMEOUT)
         self._timer.timeout.connect(self.on_timeout)
 
-        if os.name == 'nt':  # pragma: no cover
+        if utils.is_windows:  # pragma: no cover
             self._atime_timer = None
         else:
             self._atime_timer = usertypes.Timer(self, 'ipc-atime')
             self._atime_timer.setInterval(ATIME_INTERVAL)
             self._atime_timer.timeout.connect(self.update_atime)
-            self._atime_timer.setTimerType(Qt.VeryCoarseTimer)
+            self._atime_timer.setTimerType(Qt.TimerType.VeryCoarseTimer)
 
-        self._server = QLocalServer(self)
+        self._server: Optional[QLocalServer] = QLocalServer(self)
         self._server.newConnection.connect(self.handle_connection)
 
         self._socket = None
         self._old_socket = None
-        self._socketopts_ok = os.name == 'nt'
-        if self._socketopts_ok:  # pragma: no cover
-            # If we use setSocketOptions on Unix with Qt < 5.4, we get a
-            # NameError while listening...
+
+        if utils.is_windows:  # pragma: no cover
+            # As a WORKAROUND for a Qt bug, we can't use UserAccessOption on Unix. If we
+            # do, we don't get an AddressInUseError anymore:
+            # https://bugreports.qt.io/browse/QTBUG-48635
+            #
+            # Thus, we only do so on Windows, and handle permissions manually in
+            # listen() on Linux.
             log.ipc.debug("Calling setSocketOptions")
-            self._server.setSocketOptions(QLocalServer.UserAccessOption)
+            self._server.setSocketOptions(QLocalServer.SocketOption.UserAccessOption)
         else:  # pragma: no cover
             log.ipc.debug("Not calling setSocketOptions")
 
@@ -201,25 +203,19 @@ class IPCServer(QObject):
 
     def listen(self):
         """Start listening on self._socketname."""
+        assert self._server is not None
         log.ipc.debug("Listening as {}".format(self._socketname))
         if self._atime_timer is not None:  # pragma: no branch
             self._atime_timer.start()
         self._remove_server()
         ok = self._server.listen(self._socketname)
         if not ok:
-            if self._server.serverError() == QAbstractSocket.AddressInUseError:
+            if self._server.serverError() == QAbstractSocket.SocketError.AddressInUseError:
                 raise AddressInUseError(self._server)
-            else:
-                raise ListenError(self._server)
-        if not self._socketopts_ok:  # pragma: no cover
-            # If we use setSocketOptions on Unix with Qt < 5.4, we get a
-            # NameError while listening.
-            # (see b135569d5c6e68c735ea83f42e4baf51f7972281)
-            #
-            # Also, we don't get an AddressInUseError with Qt 5.5:
-            # https://bugreports.qt.io/browse/QTBUG-48635
-            #
-            # This means we only use setSocketOption on Windows...
+            raise ListenError(self._server)
+
+        if not utils.is_windows:  # pragma: no cover
+            # WORKAROUND for QTBUG-48635, see the comment in __init__ for details.
             try:
                 os.chmod(self._server.fullServerName(), 0o700)
             except FileNotFoundError:
@@ -239,37 +235,44 @@ class IPCServer(QObject):
         log.ipc.debug("Socket 0x{:x}: error {}: {}".format(
             id(self._socket), self._socket.error(),
             self._socket.errorString()))
-        if err != QLocalSocket.PeerClosedError:
+        if err != QLocalSocket.LocalSocketError.PeerClosedError:
             raise SocketError("handling IPC connection", self._socket)
 
     @pyqtSlot()
     def handle_connection(self):
         """Handle a new connection to the server."""
-        if self.ignored:
+        if self.ignored or self._server is None:
             return
         if self._socket is not None:
             log.ipc.debug("Got new connection but ignoring it because we're "
                           "still handling another one (0x{:x}).".format(
                               id(self._socket)))
             return
-        socket = self._server.nextPendingConnection()
+        socket = qtutils.add_optional(self._server.nextPendingConnection())
         if socket is None:
             log.ipc.debug("No new connection to handle.")
             return
         log.ipc.debug("Client connected (socket 0x{:x}).".format(id(socket)))
-        self._timer.start()
         self._socket = socket
+        self._timer.start()
         socket.readyRead.connect(self.on_ready_read)
         if socket.canReadLine():
             log.ipc.debug("We can read a line immediately.")
             self.on_ready_read()
-        socket.error.connect(self.on_error)
-        if socket.error() not in [QLocalSocket.UnknownSocketError,
-                                  QLocalSocket.PeerClosedError]:
+
+        socket.errorOccurred.connect(self.on_error)
+
+        # FIXME:v4 Ignore needed due to overloaded signal/method in Qt 5
+        socket_error = socket.error()  # type: ignore[operator,unused-ignore]
+        if socket_error not in [
+            QLocalSocket.LocalSocketError.UnknownSocketError,
+            QLocalSocket.LocalSocketError.PeerClosedError
+        ]:
             log.ipc.debug("We got an error immediately.")
-            self.on_error(socket.error())
+            self.on_error(socket_error)
+
         socket.disconnected.connect(self.on_disconnected)
-        if socket.state() == QLocalSocket.UnconnectedState:
+        if socket.state() == QLocalSocket.LocalSocketState.UnconnectedState:
             log.ipc.debug("Socket was disconnected immediately.")
             self.on_disconnected()
 
@@ -279,6 +282,8 @@ class IPCServer(QObject):
         log.ipc.debug("Client disconnected from socket 0x{:x}.".format(
             id(self._socket)))
         self._timer.stop()
+        if self._old_socket is not None:
+            self._old_socket.deleteLater()
         self._old_socket = self._socket
         self._socket = None
         # Maybe another connection is waiting.
@@ -286,14 +291,15 @@ class IPCServer(QObject):
 
     def _handle_invalid_data(self):
         """Handle invalid data we got from a QLocalSocket."""
+        assert self._socket is not None
         log.ipc.error("Ignoring invalid IPC data from socket 0x{:x}.".format(
             id(self._socket)))
         self.got_invalid_data.emit()
-        self._socket.error.connect(self.on_error)
+        self._socket.errorOccurred.connect(self.on_error)
         self._socket.disconnectFromServer()
 
     def _handle_data(self, data):
-        """Handle data (as bytes) we got from on_ready_ready_read."""
+        """Handle data (as bytes) we got from on_ready_read."""
         try:
             decoded = data.decode('utf-8')
         except UnicodeDecodeError:
@@ -340,35 +346,51 @@ class IPCServer(QObject):
 
         self.got_args.emit(args, target_arg, cwd)
 
-    @pyqtSlot()
-    def on_ready_read(self):
-        """Read json data from the client."""
+    def _get_socket(self, warn=True):
+        """Get the current socket for on_ready_read.
+
+        Arguments:
+            warn: Whether to warn if no socket was found.
+        """
         if self._socket is None:  # pragma: no cover
             # This happens when doing a connection while another one is already
             # active for some reason.
             if self._old_socket is None:
-                log.ipc.warning("In on_ready_read with None socket and "
-                                "old_socket!")
-                return
-            log.ipc.debug("In on_ready_read with None socket!")
+                if warn:
+                    log.ipc.warning("In _get_socket with None socket and old_socket!")
+                return None
+            log.ipc.debug("In _get_socket with None socket!")
             socket = self._old_socket
         else:
             socket = self._socket
+
+        if sip.isdeleted(socket):  # pragma: no cover
+            log.ipc.warning("Ignoring deleted IPC socket")
+            return None
+
+        return socket
+
+    @pyqtSlot()
+    def on_ready_read(self):
+        """Read json data from the client."""
         self._timer.stop()
+
+        socket = self._get_socket()
         while socket is not None and socket.canReadLine():
             data = bytes(socket.readLine())
             self.got_raw.emit(data)
             log.ipc.debug("Read from socket 0x{:x}: {!r}".format(
                 id(socket), data))
             self._handle_data(data)
-        self._timer.start()
+            socket = self._get_socket(warn=False)
+
+        if self._socket is not None:
+            self._timer.start()
 
     @pyqtSlot()
     def on_timeout(self):
         """Cancel the current connection if it was idle for too long."""
-        if self._socket is None:  # pragma: no cover
-            log.ipc.debug("on_timeout got called with None socket!")
-            return
+        assert self._socket is not None
         log.ipc.error("IPC connection timed out "
                       "(socket 0x{:x}).".format(id(self._socket)))
         self._socket.disconnectFromServer()
@@ -389,20 +411,38 @@ class IPCServer(QObject):
         access time timestamp modified at least once every 6 hours of monotonic
         time or the 'sticky' bit should be set on the file.
         """
+        assert self._server is not None
         path = self._server.fullServerName()
         if not path:
             log.ipc.error("In update_atime with no server path!")
             return
-        log.ipc.debug("Touching {}".format(path))
-        os.utime(path)
 
+        log.ipc.debug("Touching {}".format(path))
+
+        try:
+            os.utime(path)
+        except OSError:
+            log.ipc.exception("Failed to update IPC socket, trying to "
+                              "re-listen...")
+            self._server.close()
+            self.listen()
+
+    @pyqtSlot()
     def shutdown(self):
         """Shut down the IPC server cleanly."""
+        if self._server is None:
+            # We can get called twice when using :restart -- there, IPC is shut down
+            # early to avoid processing new connections while shutting down, and then
+            # we get called again when the application is about to quit.
+            return
+
         log.ipc.debug("Shutting down IPC (socket 0x{:x})".format(
             id(self._socket)))
+
         if self._socket is not None:
             self._socket.deleteLater()
             self._socket = None
+
         self._timer.stop()
         if self._atime_timer is not None:  # pragma: no branch
             self._atime_timer.stop()
@@ -410,46 +450,14 @@ class IPCServer(QObject):
                 self._atime_timer.timeout.disconnect(self.update_atime)
             except TypeError:
                 pass
+
         self._server.close()
         self._server.deleteLater()
         self._remove_server()
+        self._server = None
 
 
-def _has_legacy_server(name):
-    """Check if there is a legacy server.
-
-    Args:
-        name: The name to try to connect to.
-
-    Return:
-        True if there is a server with the given name, False otherwise.
-    """
-    socket = QLocalSocket()
-    log.ipc.debug("Trying to connect to {}".format(name))
-    socket.connectToServer(name)
-
-    err = socket.error()
-
-    if err != QLocalSocket.UnknownSocketError:
-        log.ipc.debug("Socket error: {} ({})".format(
-            socket.errorString(), err))
-
-    os_x_fail = (sys.platform == 'darwin' and
-                 socket.errorString() == 'QLocalSocket::connectToServer: '
-                                         'Unknown error 38')
-
-    if err not in [QLocalSocket.ServerNotFoundError,
-                   QLocalSocket.ConnectionRefusedError] and not os_x_fail:
-        return True
-
-    socket.disconnectFromServer()
-    if socket.state() != QLocalSocket.UnconnectedState:
-        socket.waitForDisconnected(CONNECT_TIMEOUT)
-    return False
-
-
-def send_to_running_instance(socketname, command, target_arg, *,
-                             legacy_name=None, socket=None):
+def send_to_running_instance(socketname, command, target_arg, *, socket=None):
     """Try to send a commandline to a running instance.
 
     Blocks for CONNECT_TIMEOUT ms.
@@ -459,7 +467,6 @@ def send_to_running_instance(socketname, command, target_arg, *,
         command: The command to send to the running instance.
         target_arg: --target command line argument
         socket: The socket to read data from, or None.
-        legacy_name: The legacy name to first try to connect to.
 
     Return:
         True if connecting was successful, False if no connection was made.
@@ -467,13 +474,8 @@ def send_to_running_instance(socketname, command, target_arg, *,
     if socket is None:
         socket = QLocalSocket()
 
-    if legacy_name is not None and _has_legacy_server(legacy_name):
-        name_to_use = legacy_name
-    else:
-        name_to_use = socketname
-
-    log.ipc.debug("Connecting to {}".format(name_to_use))
-    socket.connectToServer(name_to_use)
+    log.ipc.debug("Connecting to {}".format(socketname))
+    socket.connectToServer(socketname)
 
     connected = socket.waitForConnected(CONNECT_TIMEOUT)
     if connected:
@@ -492,28 +494,26 @@ def send_to_running_instance(socketname, command, target_arg, *,
         log.ipc.debug("Writing: {!r}".format(data))
         socket.writeData(data)
         socket.waitForBytesWritten(WRITE_TIMEOUT)
-        if socket.error() != QLocalSocket.UnknownSocketError:
+        if socket.error() != QLocalSocket.LocalSocketError.UnknownSocketError:
             raise SocketError("writing to running instance", socket)
-        else:
-            socket.disconnectFromServer()
-            if socket.state() != QLocalSocket.UnconnectedState:
-                socket.waitForDisconnected(CONNECT_TIMEOUT)
-            return True
+        socket.disconnectFromServer()
+        if socket.state() != QLocalSocket.LocalSocketState.UnconnectedState:
+            socket.waitForDisconnected(CONNECT_TIMEOUT)
+        return True
     else:
-        if socket.error() not in [QLocalSocket.ConnectionRefusedError,
-                                  QLocalSocket.ServerNotFoundError]:
+        if socket.error() not in [QLocalSocket.LocalSocketError.ConnectionRefusedError,
+                                  QLocalSocket.LocalSocketError.ServerNotFoundError]:
             raise SocketError("connecting to running instance", socket)
-        else:
-            log.ipc.debug("No existing instance present (error {})".format(
-                socket.error()))
-            return False
+        log.ipc.debug("No existing instance present ({})".format(
+            debug.qenum_key(QLocalSocket, socket.error())))
+        return False
 
 
 def display_error(exc, args):
     """Display a message box with an IPC error."""
     error.handle_fatal_exc(
-        exc, args, "Error while connecting to running instance!",
-        post_text="Maybe another instance is running but frozen?")
+        exc, "Error while connecting to running instance!",
+        no_err_windows=args.no_err_windows)
 
 
 def send_or_listen(args):
@@ -526,27 +526,24 @@ def send_or_listen(args):
         The IPCServer instance if no running instance was detected.
         None if an instance was running and received our request.
     """
-    socketname = _get_socketname(args.basedir)
-    legacy_socketname = _get_socketname(args.basedir, legacy=True)
+    global server
     try:
+        socketname = _get_socketname(args.basedir)
         try:
             sent = send_to_running_instance(socketname, args.command,
-                                            args.target,
-                                            legacy_name=legacy_socketname)
+                                            args.target)
             if sent:
                 return None
             log.init.debug("Starting IPC server...")
             server = IPCServer(socketname)
             server.listen()
-            objreg.register('ipc-server', server)
             return server
-        except AddressInUseError as e:
+        except AddressInUseError:
             # This could be a race condition...
             log.init.debug("Got AddressInUseError, trying again.")
             time.sleep(0.5)
             sent = send_to_running_instance(socketname, args.command,
-                                            args.target,
-                                            legacy_name=legacy_socketname)
+                                            args.target)
             if sent:
                 return None
             else:
