@@ -18,8 +18,8 @@ import enum
 import pathlib
 import operator
 import contextlib
-from typing import (Any, AnyStr, TYPE_CHECKING, BinaryIO, IO, Iterator,
-                    Optional, Union, Tuple, Protocol, cast, TypeVar)
+from typing import (Any, TYPE_CHECKING, BinaryIO, IO, Iterator, Literal,
+                    Optional, Union, Tuple, Protocol, cast, overload, TypeVar)
 
 from qutebrowser.qt import machinery, sip
 from qutebrowser.qt.core import (qVersion, QEventLoop, QDataStream, QByteArray,
@@ -80,10 +80,25 @@ def version_check(version: str,
                   compiled: bool = True) -> bool:
     """Check if the Qt runtime version is the version supplied or newer.
 
+    By default this function will check `version` against:
+
+    1. the runtime Qt version (from qVersion())
+    2. the Qt version that PyQt was compiled against (from QT_VERSION_STR)
+    3. the PyQt version (from PYQT_VERSION_STR)
+
+    With `compiled=False` only the runtime Qt version (1) is checked.
+
+    You can often run older PyQt versions against newer Qt versions, but you
+    won't be able to access any APIs that were only added in the newer Qt
+    version. So if you want to check if a new feature is supported, use the
+    default behavior. If you just want to check the underlying Qt version,
+    pass `compiled=False`.
+
     Args:
         version: The version to check against.
         exact: if given, check with == instead of >=
-        compiled: Set to False to not check the compiled version.
+        compiled: Set to False to not check the compiled Qt version or the
+          PyQt version.
     """
     if compiled and exact:
         raise ValueError("Can't use compiled=True with exact=True!")
@@ -123,6 +138,11 @@ def is_single_process() -> bool:
     return '--single-process' in args
 
 
+def is_wayland() -> bool:
+    """Check if we are running on Wayland."""
+    return objects.qapp.platformName() in ["wayland", "wayland-egl"]
+
+
 def check_overflow(arg: int, ctype: str, fatal: bool = True) -> int:
     """Check if the given argument is in bounds for the given type.
 
@@ -216,12 +236,32 @@ def deserialize_stream(stream: QDataStream, obj: _QtSerializableType) -> None:
     check_qdatastream(stream)
 
 
+@overload
+@contextlib.contextmanager
+def savefile_open(
+        filename: str,
+        binary: Literal[False] = ...,
+        encoding: str = 'utf-8'
+) -> Iterator[IO[str]]:
+    ...
+
+
+@overload
+@contextlib.contextmanager
+def savefile_open(
+        filename: str,
+        binary: Literal[True],
+        encoding: str = 'utf-8'
+) -> Iterator[IO[bytes]]:
+    ...
+
+
 @contextlib.contextmanager
 def savefile_open(
         filename: str,
         binary: bool = False,
         encoding: str = 'utf-8'
-) -> Iterator[IO[AnyStr]]:
+) -> Iterator[Union[IO[str], IO[bytes]]]:
     """Context manager to easily use a QSaveFile."""
     f = QSaveFile(filename)
     cancelled = False
@@ -233,7 +273,7 @@ def savefile_open(
         dev = cast(BinaryIO, PyQIODevice(f))
 
         if binary:
-            new_f: IO[Any] = dev  # FIXME:mypy Why doesn't AnyStr work?
+            new_f: Union[IO[str], IO[bytes]] = dev
         else:
             new_f = io.TextIOWrapper(dev, encoding=encoding)
 
